
% **************************************** October 2016  ****************************************
% ***********************************************************************************************
% ** This code has been developed by Eduardo Perez-Pellitero @ Leibniz University of Hannover. **
% **       Please refer to the wiki for proper instructions on installation and usage.         **
% *****************************************  Enjoy ! ********************************************
% ***********************************************************************************************

%%% Include path
addpath ext sources/matlab bin

%%% SR PARAMETERS %%%
config.upsample_factor = 2;     % Choose between 2, 3 and 4.
config.n_atoms = int32(1024);   % Choose between 32 and 1024.
config.n_spheres = int32(1);    % Number of hyperspheres - only training 
                                %   for exhaustive search us for now.
% Match lambda and neigborhood size value depending on the dictionary size
if(config.n_atoms == 32)
    config.lambda =0.25;
    config.neighborhood_size = 42000;
elseif (config.n_atoms ==1024)
    config.lambda =0.18;
    config.neighborhood_size = 2750;
else
    fprintf('Wrong dictionary size. Please select 32 or 1024.');
end

regressors_filename = ['UpscalerDatax' num2str(config.upsample_factor) '_' num2str(config.n_atoms) 'atoms_' num2str(config.neighborhood_size) 'neig_' num2str(config.lambda) 'lambda.mat'];

config.folder_path = 'Set5';
config.im_ext = '*.bmp';
config.filenames = glob(config.folder_path, config.im_ext);
config.id = ['PsycoSuperRes_x' num2str(config.upsample_factor) '_' num2str(config.n_atoms) 'atoms'];
upscaler_data=load(['upscaler_data/' regressors_filename]);
upscaler_data.upsample_factor = int32(config.upsample_factor);
fprintf(['This run is operating with an upscaling factor of x' num2str(config.upsample_factor) ' and ' num2str(config.n_atoms) ' atoms.\n']);


if ~exist(['Results_' config.folder_path '_' config.id], 'dir')
    mkdir(['Results_' config.folder_path '_' config.id]);
end
times = zeros (numel(config.filenames, 2));
PSNRs = zeros (numel(config.filenames, 2));
names = {'Bicubic', 'PSyCo'};

%%% Initialize PsycoSuperRes class


PsycoSuperRes_upscaler = PsycoSuperRes(upscaler_data.ordered_anchor_feats, upscaler_data.ordered_regressors, config.n_atoms, upscaler_data.GR, upscaler_data.pstep, upscaler_data.border, upscaler_data.upsample_factor, upscaler_data.P, config.n_spheres, upscaler_data.Thr,...
    upscaler_data.bucket_sizes, upscaler_data.acum_bucket_sizes, upscaler_data.patch_size, upscaler_data.dct_matrix );


% const float * anch_features, const float *regressors, const int n_regressors, const float *global_regressor, const int step, const int border,
%                              const int upscaling_factor, const float *pivots, const int n_spheres, const float *thresholds, const int *bucket_sizes, const int *acum_sizes,
%                              const int patch_side, const float * bases)

%%% Text mode progress bar
fprintf('Starting               [                    ] ... \n');

fprintf(['Running PsycoSuperRes  [']);
counter_step = 20 / numel (config.filenames);
counter = 0;

%%% IMAGE SUPER-RESOLUTION %%%
for image_it = 1 : numel (config.filenames)
    %%% Load and crop image %%%
    f = config.filenames{image_it};
    [p, n, x] = fileparts(f);
    img = imread(f);
    if (size(img, 3) == 3) %
        img = rgb2ycbcr(img);
        %imgCB = im2single(X(:,:,2));
        %imgCR = im2single(X(:,:,3));
        img = img(:, :, 1);
    end
    img = im2single(img); % to reduce memory usage
    % Crop
    im_sz = size(img);
    im_sz = im_sz - mod(im_sz, config.upsample_factor);
    img = img (1:im_sz(1), 1:im_sz(2));
    
    img_low = imresize (img, 1/config.upsample_factor, 'bicubic');
    
    PSyCo_t = tic;
    img_up = imresize (img_low, config.upsample_factor, 'bicubic');
    img_ibp = ibp(img_low, img_up, 1e-8, 2);
    img_PSyCo = PsycoSuperRes_upscaler.upscaleImagePar(img_ibp);
    times(image_it,2) = toc(PSyCo_t);
    
    img_PSyCo = shave(img_PSyCo, upscaler_data.border );
    
    bicubic = tic;
    img_bic = imresize (img_low, config.upsample_factor, 'bicubic');
    times(image_it, 1) = toc(bicubic);
    img_bic = shave(img_bic, upscaler_data.border);
    
    %imshow(PSyCo_result) % Show figure with the PSyCo upscaled image.
    PSNRs(image_it, 1) = PSNR(shave(img, upscaler_data.border), img_bic);
    PSNRs(image_it, 2) = PSNR(shave(img, upscaler_data.border), img_PSyCo);
    
    
    
    imwrite(img_PSyCo, ['Results_' config.folder_path '_' config.id '/PSyCo_' n '.png']);
    imwrite(img_bic, ['Results_' config.folder_path '_' config.id '/bicubic_' n '.png']); % Writting bicubic images, comments if not desired
    
    %%% Update progress bar
    counter = counter + counter_step;
    while (counter >= 1.0)
        fprintf('=');
        counter = counter - 1.0;
    end
    
end % for every image in the folder
fprintf('] Done. \n');
fprintf(['\nThe average PSNR for PSyCo vs bicubic is %2.4f dB vs %2.4f dB.\n'], mean(PSNRs(:,2)),mean(PSNRs(:,1)))
fprintf(['The average time for PSyCo vs bicubic is %1.4fs  vs %1.4fs s.\n'], mean(times(:,2)), mean(times(:,1)))


clear PsycoSuperRes_upscaler;
save( ['Results_' config.folder_path '_' config.id '/PSNR_and_time.mat'],'PSNRs','times')
rmpath ext sources/matlab bin
fprintf('Have a nice day!\n')


