#include "Psyco.h"
#include <cmath>
#include <cstring>
#include <omp.h>

//Default Constructor - Not very useful...
//Psyco::Psyco()
//{
//    this->patch_dim=0;
//    this->t_bases = new float [patch_dim*3]();
//}
Psyco::Psyco(const int p_side,  const float * bases)
{
    this->patch_side=p_side;
    this->patch_dim = p_side*p_side;
    this->t_bases = new float [patch_dim*3];
    for (int i=0; i < patch_dim*3 ; ++i)
        this->t_bases[i] = bases[i];
}
//Destructor deallocates the memory for the bases
Psyco::~Psyco()
{
    delete[] this->t_bases;
    //mexPrintf("It has called the destructor(PSyCo class)");
}

void Psyco::obtainTransfElement(bool * transf_element, const float * patch)
{
    float dctA[3]={0};
    float aux_dct=0.0f;
    // Force 0-initialization
    memset(transf_element, false, sizeof(bool)*4);
    // Projection on the base space
    matVecMultiplication(t_bases, patch, dctA, patch_dim, 3);


    //Transpose
    if (fabs(dctA[1]) > fabs(dctA[0]))
    {
        transf_element[0]=true;

        aux_dct = dctA[0];
        dctA[0] = dctA[1];
        dctA[1] = aux_dct;
    }

    //Fix antipodal
    if(dctA[2] < 0.0f)//row_it = 2, col_it = 1
    {
        transf_element[1]=true;
        dctA[0] = -dctA[0];
        dctA[1] = -dctA[1];
        dctA[2] = -dctA[2];
    }// end of if needs antipodal fix

    //Horizontal flip
    if(dctA[1] < 0)//row_it = 0, col_it = 1
    {
        transf_element[2]=true;
    }//If it needs horizontal flip

    //Vertical flip
    if(dctA[0] < 0)
    {
        transf_element[3]=true;
    }//If it needs vertical flip

}// end of obtainTransfElement

void Psyco::applyTransfElement (float * transf_patch, const bool * transf_element, const float * patch)
{
    float * const buffA = new float [patch_dim];
    float * const buffB = new float [patch_dim];
    float * input_ptr = buffA,
            * output_ptr = buffB,
            * temp_ptr=buffA;

    memcpy(buffA, patch, sizeof(float)*patch_dim);

    if (transf_element[0]) // DO TRANSPOSE
    {
        for (int row_it = 0; row_it < patch_side; ++row_it)
        {
            for(int col_it =0; col_it < patch_side; ++col_it)
            {
                output_ptr[row_it+col_it*patch_side] = input_ptr[col_it+row_it*patch_side];

            }//end for each col
        }//end for each row
        //Update pointers to correct buffers
        temp_ptr = input_ptr;
        input_ptr = output_ptr;
        output_ptr = temp_ptr;
    }//do transpose


    if (transf_element[1]) // DO antipodal fix
    {
        for(int row_it = 0; row_it<patch_side;++row_it)
        {
            for (int col_it = 0; col_it < patch_side; ++col_it)
            {
                input_ptr[row_it+col_it*patch_side] = -input_ptr[row_it+col_it*patch_side];
            }
        }

    }// end of if antipodal fix

    if (transf_element[2]) // DO HORIZONTAL FLIP
    {
        for(int row_it = 0; row_it<patch_side;++row_it)
        {

            for (int col_it = 0; col_it < patch_side; ++col_it)
            {
                output_ptr[row_it+col_it*patch_side] = input_ptr[row_it+(patch_side - col_it -1)*patch_side];
            }
        }
        temp_ptr = input_ptr;
        input_ptr = output_ptr;
        output_ptr = temp_ptr;

    }

    if (transf_element[3]) // DO VERTICAL FLIP
    {
        for(int col_it = 0; col_it < patch_side;++col_it)
        {
            for (int row_it = 0; row_it < patch_side; ++row_it)
            {
                output_ptr[row_it+col_it*patch_side] = input_ptr[(patch_side - row_it -1)+col_it*patch_side];
            }
        }
        temp_ptr = input_ptr;
        input_ptr = output_ptr;
        output_ptr = temp_ptr;
    } // DO vertical flip


    memcpy(transf_patch, input_ptr, sizeof(float)*patch_dim);
    delete [] buffA;
    delete [] buffB;
}// end of applyTransfElement

void Psyco::revertTransfElement (float * original_patch, const bool * transf_element, const float * transf_patch)
{
    float * const buffA = new float [patch_dim];
    float * const buffB = new float [patch_dim];
    float * input_ptr = buffA,
            * output_ptr = buffB,
            * temp_ptr=buffA;

    memcpy(buffA, transf_patch, sizeof(float)*patch_dim);

    if (transf_element[3]) // DO VERTICAL FLIP
    {
        for(int col_it = 0; col_it < patch_side;++col_it)
        {
            for (int row_it = 0; row_it < patch_side; ++row_it)
            {
                output_ptr[row_it+col_it*patch_side] = input_ptr[(patch_side - row_it -1)+col_it*patch_side];
            }
        }
        temp_ptr = input_ptr;
        input_ptr = output_ptr;
        output_ptr = temp_ptr;
    } // DO vertical flip

    if (transf_element[2]) // DO HORIZONTAL FLIP
    {
        for(int row_it = 0; row_it<patch_side;++row_it)
        {

            for (int col_it = 0; col_it < patch_side; ++col_it)
            {
                output_ptr[row_it+col_it*patch_side] = input_ptr[row_it+(patch_side - col_it -1)*patch_side];
            }
        }
        temp_ptr = input_ptr;
        input_ptr = output_ptr;
        output_ptr = temp_ptr;

    }

    if (transf_element[1]) // DO antipodal fix
    {
        for(int row_it = 0; row_it<patch_side;++row_it)
        {
            for (int col_it = 0; col_it < patch_side; ++col_it)
            {
                input_ptr[row_it+col_it*patch_side] = -input_ptr[row_it+col_it*patch_side];
            }
        }

    }// end of if antipodal fix

    if (transf_element[0]) // DO TRANSPOSE
    {
        for (int row_it = 0; row_it < patch_side; ++row_it)
        {
            for(int col_it =0; col_it < patch_side; ++col_it)
            {
                output_ptr[row_it+col_it*patch_side] = input_ptr[col_it+row_it*patch_side];

            }//end for each col
        }//end for each row
        //Update pointers to correct buffers
        temp_ptr = input_ptr;
        input_ptr = output_ptr;
        output_ptr = temp_ptr;
    }//do transpose


    memcpy(original_patch, input_ptr, sizeof(float)*patch_dim);
    delete [] buffA;
    delete [] buffB;
}// end of applyTransfElement

void Psyco::applyTransfPar (float * transf_patches, bool * transf_elements, const float * patches, const int n_patches)
{
#pragma omp parallel for
    for (int parallel_it = 0; parallel_it < n_patches; ++parallel_it)
    {
        this->obtainTransfElement(&transf_elements[4*parallel_it], &patches[patch_dim*parallel_it]);
        this->applyTransfElement(&transf_patches[patch_dim*parallel_it],&transf_elements[4*parallel_it],&patches[patch_dim*parallel_it]);
    } // end of parallel for
}

void Psyco::revertTransfPar (float * reverted_patches, const bool * transf_elements, const float * transf_patches, const int n_patches)
{
#pragma omp parallel for
    for (int parallel_it = 0; parallel_it < n_patches; ++parallel_it)
    {
        this->revertTransfElement(&reverted_patches[patch_dim*parallel_it], &transf_elements[4*parallel_it],&transf_patches[patch_dim*parallel_it]);
    } // end of parallel for
}


void Psyco::matVecMultiplication(const float * matrix, const float * vector, float *x, const int matrix_rows, const int vector_dim)
{
    for( int out = 0; out < vector_dim; ++out )
    {

        for( int in = 0; in < matrix_rows; ++in )
        {
            x[out] += matrix[out + in * vector_dim] * vector[in];
        }
    }
}//end of matVecMultiplication
