#include "mex.h"
#include "class_handle.hpp"
#include "PsycoSuperRes.h"

// The class that we are interfacing to

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{	
    // Get the command string
    char cmd[64];
    if (nrhs < 1 || mxGetString(prhs[0], cmd, sizeof(cmd)))
        mexErrMsgTxt("First input should be a command string less than 64 characters long.");

    // New
    if (!strcmp("new", cmd)) {
        // Check parameters
        if (nlhs != 1)
            mexErrMsgTxt("New: One output expected.");
        //assign other inputs
        const float * anchored_features = (float *) mxGetData(prhs[1]);
        const float * regressors = (float *) mxGetData(prhs[2]);
        const int n_regressors = (int) mxGetScalar(prhs[3]);
        const float * global_regressor = (float *) mxGetData(prhs[4]);

        const int step = (int) mxGetScalar(prhs[5]);
        const int border = (int) mxGetScalar(prhs[6]);
        const int upscaling_factor = (int) mxGetScalar(prhs[7]);

        const float * pivots = (float *) mxGetData(prhs[8]);
        const int n_spheres = (int) mxGetScalar(prhs[9]);
        const float * thresholds = (float *) mxGetData(prhs[10]);
        const int * bucket_sizes = (int *) mxGetData(prhs[11]);
        const int * acum_sizes = (int *) mxGetData(prhs[12]);

        const int patch_side = (int) mxGetScalar(prhs[13]);
        const float * bases = (float *) mxGetData(prhs[14]);



        // Return a handle to a new C++ instance
        plhs[0] = convertPtr2Mat<PsycoSuperRes>(new PsycoSuperRes(anchored_features, regressors, n_regressors, global_regressor, step, border, upscaling_factor,
                                                                  pivots, n_spheres, thresholds, bucket_sizes, acum_sizes, patch_side, bases));
        return;
    }
    
    // Check there is a second input, which should be the class instance handle
    if (nrhs < 2)
        mexErrMsgTxt("Second input should be a class instance handle.");
    
    // Delete
    if (!strcmp("delete", cmd)) {
        // Destroy the C++ object
        destroyObject<PsycoSuperRes>(prhs[1]);
        // Warn if other commands were ignored
        if (nlhs != 0 || nrhs != 2)
            mexWarnMsgTxt("Delete: Unexpected arguments ignored.");
        return;
    }
    
    // Get the class instance pointer from the second input
    PsycoSuperRes *PsycoSuperRes_instance = convertMat2Ptr<PsycoSuperRes>(prhs[1]);
    
    // Call the various class methods
    // getDim
    if (!strcmp("upscaleImagePar", cmd)) {
        // Check parameters
        if (nlhs < 0 || nrhs < 2)
            mexErrMsgTxt("Train: Unexpected arguments.");
        // Call the method
        float * image = (float *)mxGetData(prhs[2]);
        const int n_rows = mxGetM(prhs[2]);
        const int n_cols = mxGetN(prhs[2]);

        plhs[0] = mxCreateNumericMatrix(n_rows, n_cols, mxSINGLE_CLASS, mxREAL);
        float * output_image = (float *) mxGetData(plhs[0]);

        PsycoSuperRes_instance->upscaleImagePar (output_image, image, n_rows, n_cols);
        return;
    }
    // Got here, so command not recognized
    mexErrMsgTxt("Command not recognized.");
}
